@@ -21,34 +21,44 @@ class MMDataset(Dataset):
         DATASET_MAP[args['dataset_name']]()
 
     def __init_mosi(self):
-        with open(self.args['featurePath'], 'rb') as f:
-            data = pickle.load(f)
-        if 'use_bert' in self.args and self.args['use_bert']:
+        if self.args['custom_feature']:
+            # use custom feature file extracted with MMSA-FET
+            with open(self.args['custom_feature'], 'rb') as f:
+                data = pickle.load(f)
+        else:
+            # use deault feature file specified in config file
+            with open(self.args['featurePath'], 'rb') as f:
+                data = pickle.load(f)
+        
+        if self.args.get('use_bert', None):
             self.text = data[self.mode]['text_bert'].astype(np.float32)
+            self.args['feature_dims'][0] = 768
         else:
             self.text = data[self.mode]['text'].astype(np.float32)
-        self.vision = data[self.mode]['vision'].astype(np.float32)
+            self.args['feature_dims'][0] = self.text.shape[2]
         self.audio = data[self.mode]['audio'].astype(np.float32)
+        self.args['feature_dims'][1] = self.audio.shape[2]
+        self.vision = data[self.mode]['vision'].astype(np.float32)
+        self.args['feature_dims'][2] = self.vision.shape[2]
         self.raw_text = data[self.mode]['raw_text']
         self.ids = data[self.mode]['id']
 
-        # use custom features
-        use_custom_features = (self.args['feature_T'] != "" or self.args['feature_A'] != "" or self.args['feature_V'] != "")
-        if self.args['feature_T'] != "":
+        # Overide with custom modality features
+        if self.args['feature_T']:
             with open(self.args['feature_T'], 'rb') as f:
                 data_T = pickle.load(f)
-            if 'use_bert' in self.args and self.args['use_bert']:
+            if self.args.get('use_bert', None):
                 self.text = data_T[self.mode]['text_bert'].astype(np.float32)
-                self.args['feature_dims'][0] = 768 # TODO: fix this
+                self.args['feature_dims'][0] = 768
             else:
                 self.text = data_T[self.mode]['text'].astype(np.float32)
                 self.args['feature_dims'][0] = self.text.shape[2]
-        if self.args['feature_A'] != "":
+        if self.args['feature_A']:
             with open(self.args['feature_A'], 'rb') as f:
                 data_A = pickle.load(f)
             self.audio = data_A[self.mode]['audio'].astype(np.float32)
             self.args['feature_dims'][1] = self.audio.shape[2]
-        if self.args['feature_V'] != "":
+        if self.args['feature_V']:
             with open(self.args['feature_V'], 'rb') as f:
                 data_V = pickle.load(f)
             self.vision = data_V[self.mode]['vision'].astype(np.float32)
@@ -65,17 +75,35 @@ class MMDataset(Dataset):
         logger.info(f"{self.mode} samples: {self.labels['M'].shape}")
 
         if not self.args['need_data_aligned']:
-            if self.args['feature_A'] != "":
+            if self.args['feature_A']:
                 self.audio_lengths = list(data_A[self.mode]['audio_lengths'])
             else:
                 self.audio_lengths = data[self.mode]['audio_lengths']
-            if self.args['feature_V'] != "":
+            if self.args['feature_V']:
                 self.vision_lengths = list(data_V[self.mode]['vision_lengths'])
             else:
                 self.vision_lengths = data[self.mode]['vision_lengths']
         self.audio[self.audio == -np.inf] = 0
 
-        if 'need_normalized' in self.args and self.args['need_normalized']:
+        if self.args.get('data_missing'):
+            # Currently only support unaligned data missing.
+            self.text_m, self.text_length, self.text_mask, self.text_missing_mask = self.generate_m(self.text[:,0,:], self.text[:,1,:], None,
+                                                                                        self.args.missing_rate[0], self.args.missing_seed[0], mode='text')
+            Input_ids_m = np.expand_dims(self.text_m, 1)
+            Input_mask = np.expand_dims(self.text_mask, 1)
+            Segment_ids = np.expand_dims(self.text[:,2,:], 1)
+            self.text_m = np.concatenate((Input_ids_m, Input_mask, Segment_ids), axis=1)
+
+            if self.args['need_data_aligned']:
+                self.audio_lengths = np.sum(self.text[:,1,:], axis=1, dtype=np.int32)
+                self.vision_lengths = np.sum(self.text[:,1,:], axis=1, dtype=np.int32)
+
+            self.audio_m, self.audio_length, self.audio_mask, self.audio_missing_mask = self.generate_m(self.audio, None, self.audio_lengths,
+                                                                                        self.args.missing_rate[1], self.args.missing_seed[1], mode='audio')
+            self.vision_m, self.vision_length, self.vision_mask, self.vision_missing_mask = self.generate_m(self.vision, None, self.vision_lengths,
+                                                                                        self.args.missing_rate[2], self.args.missing_seed[2], mode='vision')
+
+        if self.args.get('need_normalized'):
             self.__normalize()
     
     def __init_mosei(self):
@@ -84,6 +112,28 @@ class MMDataset(Dataset):
     def __init_sims(self):
         return self.__init_mosi()
 
+    def generate_m(self, modality, input_mask, input_len, missing_rate, missing_seed, mode='text'):
+        
+        if mode == 'text':
+            input_len = np.argmin(input_mask, axis=1)
+        elif mode == 'audio' or mode == 'vision':
+            input_mask = np.array([np.array([1] * length + [0] * (modality.shape[1] - length)) for length in input_len])
+        np.random.seed(missing_seed)
+        missing_mask = (np.random.uniform(size=input_mask.shape) > missing_rate) * input_mask
+        
+        assert missing_mask.shape == input_mask.shape
+        
+        if mode == 'text':
+            # CLS SEG Token unchanged.
+            for i, instance in enumerate(missing_mask):
+                instance[0] = instance[input_len[i] - 1] = 1
+            
+            modality_m = missing_mask * modality + (100 * np.ones_like(modality)) * (input_mask - missing_mask) # UNK token: 100.
+        elif mode == 'audio' or mode == 'vision':
+            modality_m = missing_mask.reshape(modality.shape[0], modality.shape[1], 1) * modality
+        
+        return modality_m, input_len, input_mask, missing_mask
+
     def __truncate(self):
         # NOTE: truncate input to specific length.
         def do_truncate(modal_features, length):
@@ -150,6 +200,18 @@ class MMDataset(Dataset):
         if not self.args['need_data_aligned']:
             sample['audio_lengths'] = self.audio_lengths[index]
             sample['vision_lengths'] = self.vision_lengths[index]
+        if self.args.get('data_missing'):
+            sample['text_m'] = torch.Tensor(self.text_m[index])
+            sample['text_missing_mask'] = torch.Tensor(self.text_missing_mask[index])
+            sample['audio_m'] = torch.Tensor(self.audio_m[index])
+            sample['audio_lengths'] = self.audio_lengths[index]
+            sample['audio_mask'] = self.audio_mask[index]
+            sample['audio_missing_mask'] = torch.Tensor(self.audio_missing_mask[index])
+            sample['vision_m'] = torch.Tensor(self.vision_m[index])
+            sample['vision_lengths'] = self.vision_lengths[index]
+            sample['vision_mask'] = self.vision_mask[index]
+            sample['vision_missing_mask'] = torch.Tensor(self.vision_missing_mask[index])
+
         return sample
 
 def MMDataLoader(args, num_workers):
