@@ -1,14 +1,12 @@
+import errno
 import gc
 import json
 import logging
-import multiprocessing as mp
 import os
 import pickle
 import random
 import time
-from multiprocessing import Pool
 from pathlib import Path
-from webbrowser import get
 
 import numpy as np
 import pandas as pd
@@ -60,38 +58,55 @@ def _set_logger(log_dir, model_name, dataset_name, verbose_level):
 
 
 def MMSA_run(
-    model_name, dataset_name, config=None, config_file="", seeds=[], is_tune=False,
-    tune_times=50, feature_T="", feature_A="", feature_V="",
-    model_save_dir="", res_save_dir="", log_dir="",
-    gpu_ids=[0], num_workers=4, verbose_level=1
+    model_name: str, dataset_name: str, config_file: str = None,
+    config: dict = None, seeds: list = [], is_tune: bool = False,
+    tune_times: int = 50, custom_feature: str = None, feature_T: str = None, 
+    feature_A: str = None, feature_V: str = None, gpu_ids: list = [0],
+    num_workers: int = 4, verbose_level: int = 1,
+    model_save_dir: str = Path().home() / "MMSA" / "saved_models",
+    res_save_dir: str = Path().home() / "MMSA" / "results",
+    log_dir: str = Path().home() / "MMSA" / "logs",
 ):
-    """
-    Main function for running the MMSA framework.
+    """Train and Test MSA models.
 
-    Parameters:
-        model_name (str): Name of model
-        dataset_name (str): Name of dataset
-        config (dict): Config dict. Used to override arguments in config_file. Ignored in tune mode.
-        config_file: Path to config file. If not specified, default config file will be used.
-        seeds (list): List of seeds. Default: [1111, 1112, 1113, 1114, 1115]
-        is_tune (bool): Whether to tune hyper parameters. Default: False
-        tune_times (int): Number of times to tune hyper parameters. Default: 50
-        feature_T (str): Path to text feature file. 
-        feature_A (str): Path to audio feature file. 
-        feature_V (str): Path to video feature file.
-        model_save_dir (str): Path to save trained models. Default: "~/MMSA/saved_models"
-        res_save_dir (str): Path to save csv results. Default: "~/MMSA/results"
-        log_dir (str): Path to save log files. Default: "~/MMSA/logs"
-        gpu_ids (list): Specify which gpus to use. If a empty list is supplied, will auto assign to the most memory-free gpu. Default: [0]
-                        Currently only support single gpu.
-        num_workers (int): Number of workers used to load data. Default: 4
-        verbose_level (int): Verbose level of stdout. 0 for error, 1 for info, 2 for debug. Default: 1
+    Given a set of hyper-parameters(via config), will train models on training
+    and validation set, then test on test set and report the results. If 
+    `is_tune` is set, will accept lists as hyper-parameters and conduct a grid
+    search to find the optimal values.
+
+    Args:
+        model_name: Name of MSA model.
+        dataset_name: Name of MSA dataset.
+        config_file: Path to config file. If not specified, default config
+            files will be used.
+        config: Config dict. Used to override arguments in config_file. 
+        seeds: List of seeds. Default: [1111, 1112, 1113, 1114, 1115]
+        is_tune: Tuning mode switch. Default: False
+        tune_times: Sets of hyper parameters to tune. Default: 50
+        custom_feature: Path to custom feature file. The custom feature should
+            contain features of all three modalities. If only one modality has
+            customized features, use `feature_*` below. 
+        feature_T: Path to text feature file. Provide an empty string to use
+            default BERT features. Default: ""
+        feature_A: Path to audio feature file. Provide an empty string to use
+            default features provided by dataset creators. Default: ""
+        feature_V: Path to video feature file. Provide an empty string to use
+            default features provided by dataset creators. Default: ""
+        gpu_ids: GPUs to use. Will assign the most memory-free gpu if an empty
+            list is provided. Default: [0]. Currently only supports single gpu.
+        num_workers: Number of workers used to load data. Default: 4
+        verbose_level: Verbose level of stdout. 0 for error, 1 for info, 2 for
+            debug. Default: 1
+        model_save_dir: Path to save trained model weights. Default: 
+            "~/MMSA/saved_models"
+        res_save_dir: Path to save csv results. Default: "~/MMSA/results"
+        log_dir: Path to save log files. Default: "~/MMSA/logs"
     """
     # Initialization
     model_name = model_name.lower()
     dataset_name = dataset_name.lower()
     
-    if config_file != "":
+    if config_file is not None:
         config_file = Path(config_file)
     else: # use default config files
         if is_tune:
@@ -99,14 +114,14 @@ def MMSA_run(
         else:
             config_file = Path(__file__).parent / "config" / "config_regression.json"
     if not config_file.is_file():
-        raise ValueError(f"Config file {str(config_file)} not found.")
-    if model_save_dir == "": # use default model save dir
+        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), config_file)
+    if model_save_dir is None: # use default model save dir
         model_save_dir = Path.home() / "MMSA" / "saved_models"
     Path(model_save_dir).mkdir(parents=True, exist_ok=True)
-    if res_save_dir == "": # use default result save dir
+    if res_save_dir is None: # use default result save dir
         res_save_dir = Path.home() / "MMSA" / "results"
     Path(res_save_dir).mkdir(parents=True, exist_ok=True)
-    if log_dir == "": # use default log save dir
+    if log_dir is None: # use default log save dir
         log_dir = Path.home() / "MMSA" / "logs"
     Path(log_dir).mkdir(parents=True, exist_ok=True)
     seeds = seeds if seeds != [] else [1111, 1112, 1113, 1114, 1115]
@@ -115,12 +130,12 @@ def MMSA_run(
     logger.info("======================================== Program Start ========================================")
     
     if is_tune: # run tune
-        setup_seed(seeds[0])
         logger.info(f"Tuning with seed {seeds[0]}")
         initial_args = get_config_tune(model_name, dataset_name, config_file)
         initial_args['model_save_path'] = Path(model_save_dir) / f"{initial_args['model_name']}-{initial_args['dataset_name']}.pth"
         initial_args['device'] = assign_gpu(gpu_ids)
         initial_args['train_mode'] = 'regression' # backward compatibility. TODO: remove all train_mode in code
+        initial_args['custom_feature'] = custom_feature
         initial_args['feature_T'] = feature_T
         initial_args['feature_A'] = feature_A
         initial_args['feature_V'] = feature_V
@@ -143,8 +158,11 @@ def MMSA_run(
             args = edict(**initial_args)
             random.seed(time.time())
             new_args = get_config_tune(model_name, dataset_name, config_file)
-            random.seed(seeds[0])
             args.update(new_args)
+            if config:
+                if config.get('model_name'):
+                    assert(config['model_name'] == args['model_name'])
+                args.update(config)
             args['cur_seed'] = i + 1
             logger.info(f"{'-'*30} Tuning [{i + 1}/{tune_times}] {'-'*30}")
             logger.info(f"Args: {args}")
@@ -154,9 +172,10 @@ def MMSA_run(
                 logger.info(f"This set of parameters has been run. Skip.")
                 time.sleep(1)
                 continue
-            has_debuged.append(cur_param)
             # actual running
+            setup_seed(seeds[0])
             result = _run(args, num_workers, is_tune)
+            has_debuged.append(cur_param)
             # save result to csv file
             if Path(csv_file).is_file():
                 df2 = pd.read_csv(csv_file)
@@ -174,10 +193,13 @@ def MMSA_run(
         args['model_save_path'] = Path(model_save_dir) / f"{args['model_name']}-{args['dataset_name']}.pth"
         args['device'] = assign_gpu(gpu_ids)
         args['train_mode'] = 'regression' # backward compatibility. TODO: remove all train_mode in code
+        args['custom_feature'] = custom_feature
         args['feature_T'] = feature_T
         args['feature_A'] = feature_A
         args['feature_V'] = feature_V
         if config: # override some arguments
+            if config.get('model_name'):
+                assert(config['model_name'] == args['model_name'])
             args.update(config)
 
         # torch.cuda.set_device() encouraged by pytorch developer, although dicouraged in the doc.
@@ -259,14 +281,87 @@ def _run(args, num_workers=4, is_tune=False, from_sena=False):
     return {"epoch_results": epoch_results, 'final_results': final_results} if from_sena else results
 
 
+def MMSA_test(
+    config: dict | str,
+    weights_path: str,
+    feature_path: str, 
+    # seeds: list = [], 
+    gpu_id: int = 0, 
+):
+    """Test MSA models on a single sample.
+
+    Load weights and configs of a saved model, input pre-extracted
+    features of a video, then get sentiment prediction results.
+
+    Args:
+        model_name: Name of MSA model.
+        config: Config dict or path to config file. 
+        weights_path: Pkl file path of saved model weights.
+        feature_path: Pkl file path of pre-extracted features.
+        gpu_id: Specify which gpu to use. Use cpu if value < 0.
+    """
+    if type(config) == str or type(config) == Path:
+        config = Path(config)
+        with open(config, 'r') as f:
+            args = json.load(f)
+    elif type(config) == dict or type(config) == edict:
+        args = config
+    else:
+        raise ValueError(f"'config' should be string or dict, not {type(config)}")
+    args['train_mode'] = 'regression' # backward compatibility.
+
+    if gpu_id < 0:
+        device = torch.device('cpu')
+    else:
+        device = torch.device(f'cuda:{gpu_id}')
+    args['device'] = device
+    with open(feature_path, "rb") as f:
+        feature = pickle.load(f)
+    args['feature_dims'] = [feature['text'].shape[1], feature['audio'].shape[1], feature['vision'].shape[1]]
+    args['seq_lens'] = [feature['text'].shape[0], feature['audio'].shape[0], feature['vision'].shape[0]]
+    model = AMIO(args)
+    model.load_state_dict(torch.load(weights_path), strict=False)
+    model.to(device)
+    model.eval()
+    with torch.no_grad():
+        if args.get('use_bert', None):
+            if type(text := feature['text_bert']) == np.ndarray:
+                text = torch.from_numpy(text).float()
+        else:
+            if type(text := feature['text']) == np.ndarray:
+                text = torch.from_numpy(text).float()
+        if type(audio := feature['audio']) == np.ndarray:
+            audio = torch.from_numpy(audio).float()
+        if type(vision := feature['vision']) == np.ndarray:
+            vision = torch.from_numpy(vision).float()
+        text = text.unsqueeze(0).to(device)
+        audio = audio.unsqueeze(0).to(device)
+        vision = vision.unsqueeze(0).to(device)
+        if args.get('need_normalized', None):
+            audio = torch.mean(audio, dim=1, keepdims=True)
+            vision = torch.mean(vision, dim=1, keepdims=True)
+        # TODO: write a do_single_test function for each model in trains
+        if args['model_name'] == 'self_mm' or args['model_name'] == 'mmim':
+            output = model(text, (audio, torch.tensor(audio.shape[1]).unsqueeze(0)), (vision, torch.tensor(vision.shape[1]).unsqueeze(0)))
+        elif args['model_name'] == 'tfr_net':
+            input_mask = torch.tensor(feature['text_bert'][1]).unsqueeze(0).to(device)
+            output, _ = model((text, text, None), (audio, audio, input_mask, None), (vision, vision, input_mask, None))
+        else:
+            output = model(text, audio, vision)
+        if type(output) == dict:
+            output = output['M']
+    return output.cpu().detach().numpy()[0][0]
+        
+
 SENA_ENABLED = True
 try:
     from datetime import datetime
     from multiprocessing import Queue
-    from sklearn.decomposition import PCA
+
     import mysql.connector
+    from sklearn.decomposition import PCA
 except ImportError:
-    logger.warning("SENA_run is not loaded due to missing dependencies. This is ok if you are not using M-SENA.")
+    logger.debug("SENA_run is not loaded due to missing dependencies. Ignore this if you are not using M-SENA Platform.")
     SENA_ENABLED = False
 
 if SENA_ENABLED:
@@ -336,18 +431,18 @@ if SENA_ENABLED:
             args['feature_A'] = feature_A
             args['feature_V'] = feature_V
             # determine feature_dims
-            if args['feature_T'] != "":
+            if args['feature_T']:
                 with open(args['feature_T'], 'rb') as f:
                     data_T = pickle.load(f)
                 if 'use_bert' in args and args['use_bert']:
                     args['feature_dims'][0] = 768
                 else:
                     args['feature_dims'][0] = data_T['valid']['text'].shape[2]
-            if args['feature_A'] != "":
+            if args['feature_A']:
                 with open(args['feature_A'], 'rb') as f:
                     data_A = pickle.load(f)
                 args['feature_dims'][1] = data_A['valid']['audio'].shape[2]
-            if args['feature_V'] != "":
+            if args['feature_V']:
                 with open(args['feature_V'], 'rb') as f:
                     data_V = pickle.load(f)
                 args['feature_dims'][2] = data_V['valid']['vision'].shape[2]
@@ -432,8 +527,10 @@ if SENA_ENABLED:
                         for name in ['Feature_t', 'Feature_a', 'Feature_v', 'Feature_f']: 
                             cur_features = []
                             for mode in select_modes:
-                                cur_features.append(features[mode][name])
-                            cur_features = np.concatenate(cur_features, axis=0)
+                                if name in features[mode]:
+                                    cur_features.append(features[mode][name])
+                            if cur_features != []:
+                                cur_features = np.concatenate(cur_features, axis=0)
                             # PCA analysis
                             if len(cur_features) != 0:
                                 pca=PCA(n_components=3, whiten=True)
@@ -562,4 +659,4 @@ if SENA_ENABLED:
         with torch.no_grad():
             outputs = model(feature_t, feature_a, feature_v)
         predict = round(float(outputs['M'].cpu().detach().squeeze()), 3)
-        return predict
\ No newline at end of file
+        return predict
