@@ -1,8 +1,9 @@
 """
-AIO -- All Trains in One
+ATIO -- All Trains in One
 """
 from .multiTask import *
 from .singleTask import *
+from .missingTask import *
 
 __all__ = ['ATIO']
 
@@ -17,6 +18,7 @@ class ATIO():
             'lf_dnn': LF_DNN,
             'graph_mfn': Graph_MFN,
             'mult': MULT,
+            'mctn': MCTN,
             'bert_mag':BERT_MAG,
             'misa': MISA,
             'mfm': MFM,
@@ -26,6 +28,8 @@ class ATIO():
             'mlmf': MLMF,
             'mlf_dnn': MLF_DNN,
             'self_mm': SELF_MM,
+            # missing-task
+            'tfr_net': TFR_NET,
         }
     
     def getTrain(self, args):
