@@ -20,10 +20,14 @@ class MTFN():
         self.metrics = MetricsTop(args.train_mode).getMetics(args.dataset_name)
 
     def do_train(self, model, dataloader, return_epoch_results=False):
+        model_params_other = [p for n, p in list(model.Model.named_parameters())[2:] if 'text_subnet' not in n and \
+                                'audio_subnet' not in n and 'video_subnet' not in n]
+
         optimizer = optim.Adam([{"params": list(model.Model.text_subnet.parameters()), "weight_decay": self.args.text_weight_decay},
                                 {"params": list(model.Model.audio_subnet.parameters()), "weight_decay": self.args.audio_weight_decay},
                                 {"params": list(model.Model.video_subnet.parameters()), "weight_decay": self.args.video_weight_decay},
-                                {"params": list(model.parameters())[:2], "lr": 0.0}],
+                                {"params": list(model.parameters())[:2], "lr": 0.0},
+                                {'params': model_params_other}],
                                 lr=self.args.learning_rate)
         # initilize results
         epochs, best_epoch = 0, 0
